from admitad.abstract_models import AbstractAdmitadPostbackEvent, AbstractAdmitadPostbackRequest
from admitad.utils import is_model_registered

__all__ = []

if not is_model_registered('admitad', 'AdmitadPostbackEvent)'):
    class AdmitadPostbackEvent(AbstractAdmitadPostbackEvent):
        pass


    __all__.append('AdmitadPostbackEvent')

if not is_model_registered('admitad', 'AdmitadPostbackRequest)'):
    class AdmitadPostbackRequest(AbstractAdmitadPostbackRequest):
        pass


    __all__.append('AdmitadPostbackRequest')



# Generated by Django 2.1.3 on 2018-11-27 13:29

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('admitad', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='admitadpostbackrequest',
            name='request_url',
            field=models.TextField(validators=[django.core.validators.URLValidator()], verbose_name='Request URL'),
        ),
    ]



# Generated by Django 2.1.3 on 2018-11-25 09:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AdmitadPostbackEvent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order_id', models.PositiveIntegerField(verbose_name='Order ID')),
                ('admitad_uid', models.CharField(max_length=64, verbose_name='Admitad UID')),
                ('campaign_code', models.CharField(max_length=64, verbose_name='Campain code')),
                ('payment_type', models.CharField(choices=[('Sale', 'Sale'), ('Lead', 'Lead')], max_length=4, verbose_name='Payment type')),
                ('action_code', models.CharField(max_length=64, verbose_name='Action code')),
                ('integration_status', models.CharField(choices=[('Created', 'Created'), ('Success', 'Success'), ('Failed', 'Failed')], default='Created', max_length=16, verbose_name='Integration status')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
            ],
            options={
                'verbose_name': 'Admitad postback event',
                'verbose_name_plural': 'Admitad postback events',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AdmitadPostbackRequest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('product_id', models.PositiveIntegerField(verbose_name='Product ID')),
                ('position_id', models.PositiveIntegerField(verbose_name='Position ID')),
                ('quantity', models.PositiveIntegerField(verbose_name='Quantity')),
                ('currency_code', models.CharField(max_length=3, verbose_name='Currency_code')),
                ('tariff_code', models.CharField(max_length=16, verbose_name='Tariff code')),
                ('price', models.DecimalField(decimal_places=2, max_digits=12, verbose_name='Price')),
                ('request_url', models.URLField(verbose_name='Request URL')),
                ('integration_status', models.CharField(choices=[('Created', 'Created'), ('Success', 'Success'), ('Failed', 'Failed')], default='Created', max_length=16, verbose_name='Integration status')),
                ('error_text', models.TextField(blank=True, null=True, verbose_name='Error text')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='requests', to='admitad.AdmitadPostbackEvent', verbose_name='Event')),
            ],
            options={
                'verbose_name': 'Admitad postback request',
                'verbose_name_plural': 'Admitad postback requests',
                'abstract': False,
            },
        ),
    ]



# Generated by Django 2.1.3 on 2018-11-27 14:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('admitad', '0002_auto_20181127_1329'),
    ]

    operations = [
        migrations.AlterField(
            model_name='admitadpostbackevent',
            name='payment_type',
            field=models.CharField(choices=[('sale', 'Sale'), ('lead', 'Lead')], max_length=4, verbose_name='Payment type'),
        ),
    ]



from django.apps import apps

try:
    import urlparse
    from urllib import urlencode
except ImportError:  # For Python 3
    import urllib.parse as urlparse
    from urllib.parse import urlencode


def add_params_to_url(url, params):
    """
    Applies params to url
    """
    url_parts = list(urlparse.urlparse(url))
    query = dict(urlparse.parse_qsl(url_parts[4]))
    query.update(params)
    url_parts[4] = urlencode(query)

    return urlparse.urlunparse(url_parts)


def is_model_registered(app_label, model_name):
    """
    Checks whether a given model is registered. This is used to only
    register admitad models if they aren't overridden by a forked app.
    """
    try:
        apps.get_registered_model(app_label, model_name)
    except LookupError:
        return False
    else:
        return True



from setuptools import setup

setup(
    name='django-admitad',
    version='1.1.1',
    packages=['admitad'],
    package_data={
        'admitad': ['migrations/*.py']
    },
    install_requires=[
        'Django>=1.11',
    ],
    url='https://github.com/Picasel/django-admitad',
    license='MIT License',
    author='k0t3n',
    author_email='k0t3n.mail@gmail.com',
    description='Django application that integrates Admitad CPA using postback requests',
    classifiers=[
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
)



import requests
from django.conf import settings

from admitad.utils import add_params_to_url
from .models import AdmitadPostbackEvent, AdmitadPostbackRequest


class Order:
    """
    Order object
    """
    items = []

    def __init__(self, internal_id, admitad_uid, action_code,
                 payment_type=AdmitadPostbackEvent.SALE, items=None):

        self.internal_id = internal_id
        self.admitad_uid = admitad_uid
        self.action_code = action_code
        self.payment_type = payment_type

        if items:
            self.items += items

    def __eq__(self, obj):
        return self.__dict__ == obj.__dict__

    def add_item(self, item):
        """
        Adds item t order
        """
        self.items.append(item)
        return True

    @property
    def full_price(self):
        """
        Returns order full price
        """
        price = 0
        for item in self.items:
            price += item.full_price

        return price

    @property
    def items_count(self):
        """
        Returns the items count in the order
        """
        return len(self.items)

    def send_postback_requests(self):
        """
        Sends postback requests to Admitad
        :return:
        """
        event = AdmitadPostbackEvent.objects.create(
            order_id=self.internal_id,
            admitad_uid=self.admitad_uid,
            campaign_code=settings.ADMITAD_COMPAIN_CODE,
            payment_type=self.payment_type,
            action_code=self.action_code,
        )

        is_failed = False
        for params in self._generate_postback_payload():
            postback_request = AdmitadPostbackRequest.objects.create(
                event=event,
                product_id=params['product_id'],
                position_id=params['position_id'],
                quantity=params['quantity'],
                currency_code=params['currency_code'],
                tariff_code=params['tariff_code'],
                price=params['price'],
                request_url=add_params_to_url(settings.ADMITAD_POSTBACK_URL, params)
            )
            try:
                response = requests.get(postback_request.request_url)
            except requests.exceptions.RequestException as error:
                is_failed = True
                postback_request.mark_as_failed(error)
            else:
                if response.status_code != 200:
                    is_failed = True
                    postback_request.mark_as_failed(response.content)

                postback_request.mark_as_success()

        if is_failed:
            event.mark_as_failed()
            return False

        else:
            event.mark_as_success()
            return True

    def _generate_postback_payload(self):
        """
        Generates payload for postback requests
        """
        base_params = {
            'uid': self.admitad_uid,
            'campaign_code': settings.ADMITAD_COMPAIN_CODE,
            'order_id': self.internal_id,
            'action_code': self.action_code,
            'position_count': self.items_count,
            'payment_type': self.payment_type,
            'postback': 1,
            'postback_key': settings.ADMITAD_POSTBACK_KEY
        }

        for position, item in enumerate(self.items, start=1):
            request_params = base_params.copy()
            request_params.update({
                'position_id': position,
                'tariff_code': item.tariff_code,
                'product_id': item.internal_id,
                'quantity': item.quantity,
                'price': item.item_price,
                'currency_code': item.currency_code
            })

            yield request_params


class Item:
    """
    Item object
    """

    def __init__(self, internal_id, item_price, tariff_code, quantity=1, currency_code='RUB'):
        self.internal_id = internal_id
        self.item_price = item_price
        self.tariff_code = tariff_code
        self.currency_code = currency_code.upper()
        self.quantity = quantity

    def __eq__(self, obj):
        return self.__dict__ == obj.__dict__

    @property
    def price(self):
        """
        Returns price per item
        """
        return self.item_price

    @property
    def full_price(self):
        """
        Returns full price including quantity
        """
        return self.item_price * self.quantity
